# -- General configuration ------------------------------------------------

import os

import riemann_client

if not os.environ.get('READTHEDOCS', None):
    import sphinx_rtd_theme
    html_theme = 'sphinx_rtd_theme'
    html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

master_doc = 'index'
exclude_patterns = ['_build']

# General information about the project.
project = u'riemann-client'
copyright = u'2014, ' + riemann_client.__author__
version = release = riemann_client.__version__

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.coverage',
    'sphinx.ext.viewcode'
]
autodoc_member_order = 'bysource'
autoclass_content = 'both'



"""Transports are used for direct communication with the Riemann server. They
are usually used inside a :py:class:`.Client`, and are used to send and receive
protocol buffer objects."""

from __future__ import absolute_import

import abc
import socket
import ssl
import struct

import riemann_client.riemann_pb2


# Default arguments
HOST = 'localhost'
PORT = 5555
TIMEOUT = None


def socket_recvall(socket, length, bufsize=4096):
    """A helper method to read of bytes from a socket to a maximum length"""
    data = b""
    while len(data) < length:
        data += socket.recv(bufsize)
    return data


class RiemannError(Exception):
    """Raised when the Riemann server returns an error message"""
    pass


class Transport(object):
    """Abstract transport definition

    Subclasses must implement the :py:meth:`.connect`, :py:meth:`.disconnect`
    and :py:meth:`.send` methods.

    Can be used as a context manager, which will call :py:meth:`.connect` on
    entry and :py:meth:`.disconnect` on exit.
    """

    __metaclass__ = abc.ABCMeta

    def __enter__(self):
        self.connect()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.disconnect()

    @abc.abstractmethod
    def connect(self):
        pass

    @abc.abstractmethod
    def disconnect(self):
        pass

    @abc.abstractmethod
    def send(self):
        pass


class SocketTransport(Transport):
    """Provides common methods for Transports that use a sockets"""

    def __init__(self, host=HOST, port=PORT):
        self.host = host
        self.port = port

    @property
    def address(self):
        """
        :returns: A tuple describing the address to connect to
        :rtype: (host, port)
        """
        return self.host, self.port

    @property
    def socket(self):
        """Returns the socket after checking it has been created"""
        if not hasattr(self, '_socket'):
            raise RuntimeError("Transport has not been connected!")
        return self._socket

    @socket.setter
    def socket(self, value):
        self._socket = value


class UDPTransport(SocketTransport):
    def connect(self):
        """Creates a UDP socket"""
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    def disconnect(self):
        """Closes the socket"""
        self.socket.close()

    def send(self, message):
        """Sends a message, but does not return a response

        :returns: None - can't receive a response over UDP
        """
        self.socket.sendto(message.SerializeToString(), self.address)
        return None


class TCPTransport(SocketTransport):
    def __init__(self, host=HOST, port=PORT, timeout=TIMEOUT):
        """Communicates with Riemann over TCP

        :param str host: The hostname to connect to
        :param int port: The port to connect to
        :param int timeout: The time in seconds to wait before raising an error
        """
        super(TCPTransport, self).__init__(host, port)
        self.timeout = timeout

    def connect(self):
        """Connects to the given host"""
        self.socket = socket.create_connection(self.address, self.timeout)

    def disconnect(self):
        """Closes the socket"""
        self.socket.close()

    def send(self, message):
        """Sends a message to a Riemann server and returns it's response

        :param message: The message to send to the Riemann server
        :returns: The response message from Riemann
        :raises RiemannError: if the server returns an error
        """
        message = message.SerializeToString()
        self.socket.sendall(struct.pack('!I', len(message)) + message)

        length = struct.unpack('!I', self.socket.recv(4))[0]
        response = riemann_client.riemann_pb2.Msg()
        response.ParseFromString(socket_recvall(self.socket, length))

        if not response.ok:
            raise RiemannError(response.error)

        return response


class TLSTransport(TCPTransport):
    def __init__(self, host=HOST, port=PORT, timeout=TIMEOUT, ca_certs=None):
        """Communicates with Riemann over TCP + TLS

        Options are the same as :py:class:`.TCPTransport` unless noted

        :param str ca_certs: Path to a CA Cert bundle used to create the socket
        """
        super(TLSTransport, self).__init__(host, port, timeout)
        self.ca_certs = ca_certs

    def connect(self):
        """Connects using :py:meth:`TLSTransport.connect` and wraps with TLS"""
        super(TLSTransport, self).connect()
        self.socket = ssl.wrap_socket(
            self.socket,
            ssl_version=ssl.PROTOCOL_TLSv1,
            cert_reqs=ssl.CERT_REQUIRED,
            ca_certs=self.ca_certs)


class BlankTransport(Transport):
    """A transport that collects events in a list, and has no connection

    Used by ``--transport none``, which is useful for testing commands without
    contacting a Riemann server. This is also used by the automated tests in
    ``riemann_client/tests/test_riemann_command.py``.
    """

    def __init__(self, *args, **kwargs):
        self.events = []

    def connect(self):
        """Creates a list to hold messages"""
        pass

    def send(self, message):
        """Adds a message to the list, returning a fake 'ok' response

        :returns: A response message with ``ok = True``
        """
        for event in message.events:
            self.events.append(event)
        reply = riemann_client.riemann_pb2.Msg()
        reply.ok = True
        return reply

    def disconnect(self):
        """Clears the list of messages"""
        pass

    def __len__(self):
        return len(self.events)



from __future__ import absolute_import

import socket
import sys
import uuid

import py.test

import riemann_client.client
import riemann_client.riemann_pb2
import riemann_client.transport

if sys.version_info >= (3,):
    basestring = str


@py.test.fixture
def client(request, string_transport):
    client = riemann_client.client.Client(transport=string_transport)
    client.transport.connect()

    @request.addfinalizer
    def disconnect():
        client.transport.disconnect()

    return client


@py.test.fixture
def unique():
    return str(uuid.uuid4())


def test_default_transport():
    assert riemann_client.client.Client()


class TestClient(object):
    def test_service(self, client):
        client.event(service='test event')
        assert 'test event' in client.transport.string.getvalue()

    def test_default_hostname(self, client, event):
        event = client.event().events[0]
        assert socket.gethostname() == event.host
        assert socket.gethostname() in client.transport.string.getvalue()

    def test_custom_hostname(self, client):
        event = client.event(host="test.example.com").events[0]
        assert "test.example.com" == event.host
        assert "test.example.com" in client.transport.string.getvalue()

    def test_tags(self, client, unique):
        client.event(tags=[unique])
        assert unique in client.transport.string.getvalue()

    def test_attributes(self, client, unique):
        client.event(attributes={'key': unique})
        assert unique in client.transport.string.getvalue()

    def test_event_cls(self, event):
        assert isinstance(event, riemann_client.riemann_pb2.Event)

    def test_query(self, client):
        assert client.query('true') == []

    def test_udp_query(self):
        transport = riemann_client.transport.UDPTransport()
        client = riemann_client.client.Client(transport)

        with py.test.raises(Exception):
            client.query('true')

    def test_events(self, client):
        message = client.events({'service': 'one'}, {'service': 'two'})
        assert message.events[0].service == 'one'
        assert message.events[1].service == 'two'

    def test_events_len(self, client):
        message = client.events({'service': 'one'}, {'service': 'two'})
        assert len(message.events) == 2


@py.test.fixture
def event(unique):
    return riemann_client.client.Client.create_event({
        'host': 'test.example.com',
        'tags': [unique],
        'attributes': {
            unique: unique
        }
    })


class TestCreateEvent(object):
    def test_host(self, event):
        assert event.host == 'test.example.com'

    def test_tags(self, event, unique):
        assert unique in event.tags

    def test_attributes(self, event, unique):
        assert event.attributes[0].key == unique
        assert event.attributes[0].value == unique


@py.test.fixture
def event_as_dict(event):
    return riemann_client.client.Client.create_dict(event)


class TestCreateDict(object):
    def test_host(self, event, event_as_dict):
        assert event.host == event_as_dict['host']

    def test_host_type(self, event_as_dict):
        assert isinstance(event_as_dict['host'], basestring)

    def test_tags(self, event, event_as_dict):
        for tag in event.tags:
            assert tag in event_as_dict['tags']

    def test_tags_type(self, event_as_dict):
        assert isinstance(event_as_dict['tags'], list)

    def test_attibutes(self, event, event_as_dict):
        for attr in event.attributes:
            assert event_as_dict['attributes'][attr.key] == attr.value

    def test_attibutes_type(self, event_as_dict):
        assert isinstance(event_as_dict['attributes'], dict)



"""Riemann command line client"""

from __future__ import absolute_import, print_function

import json
import sys

import click

import riemann_client
import riemann_client.client
import riemann_client.transport

__all__ = ['main']


class CommandLineClient(riemann_client.client.Client):
    """Prints to STDERR when an error message is recived from Riemann"""

    def __exit__(self, exc_type, exc_value, traceback):
        super(CommandLineClient, self).__exit__(exc_type, exc_value, traceback)

        if isinstance(exc_type, riemann_client.transport.RiemannError):
            click.echo("The server responded with an error: {0}".format(
                exc_value.message), file=sys.stderr)
            exit(1)


class Pair(click.ParamType):
    """A key value parameter seperated with an '=' symbol"""
    name = 'pair'

    def convert(self, value, param, ctx):
        key, value = value.split('=', 1)
        return key.strip(), value.strip()


def echo_event(data):
    """Echo a json dump of an object using click"""
    return click.echo(json.dumps(data, sort_keys=True, indent=2))


@click.group()
@click.version_option(version=riemann_client.__version__)
@click.option('--host', '-H', type=click.STRING, default='localhost',
              envvar='RIEMANN_HOST', help='Riemann server hostname.')
@click.option('--port', '-P', type=click.INT, default=5555,
              envvar='RIEMANN_PORT', help='Riemann server port.')
@click.option('--transport', '-T', 'transport_type', default='tcp',
              type=click.Choice(['udp', 'tcp', 'tls', 'none']),
              help='The protocol to use to connect to Riemann.')
@click.option('--timeout', '-I', type=click.FLOAT, default=None,
              help='Timeout for TCP based connections.')
@click.option('--ca-certs', '-C', type=click.Path(),
              help='A CA certificate bundle for TLS connections.')
@click.pass_context
def main(ctx, host, port, transport_type, timeout, ca_certs):
    """Connects to a Riemann server to send events or query the index

    By default, will attempt to contact Riemann on localhost:5555 over TCP. The
    RIEMANN_HOST and RIEMANN_PORT environment variables can be used to
    configure the host and port used. Command line parameters will override the
    environment variables.

    Use `-T none` to test commands without actually connecting to a server.
    """
    if transport_type == 'udp':
        if timeout is not None:
            ctx.fail('--timeout cannot be used with the UDP transport')
        transport = riemann_client.transport.UDPTransport(host, port)
    elif transport_type == 'tcp':
        transport = riemann_client.transport.TCPTransport(host, port, timeout)
    elif transport_type == 'tls':
        if ca_certs is None:
            ctx.fail('--ca-certs must be set when using the TLS transport')
        transport = riemann_client.transport.TLSTransport(
            host, port, timeout, ca_certs)
    elif transport_type == 'none':
        transport = riemann_client.transport.BlankTransport()

    ctx.obj = transport


@main.command()
@click.option('-T', '--time', type=click.INT,
              help="Event timestamp (unix format)")
@click.option('-S', '--state', type=click.STRING,
              help="Event state")
@click.option('-s', '--service', type=click.STRING,
              help="Event service name")
@click.option('-h', '--host', type=click.STRING,
              help="Event hostname (uses system's by default)")
@click.option('-d', '--description', type=click.STRING,
              help="Event description")
@click.option('-t', '--tag', type=click.STRING, multiple=True,
              help="Event tag (multiple)")
@click.option('-l', '--ttl', type=click.FLOAT,
              help="Event time to live in seconds")
@click.option('-a', '--attr', '--attribute', type=Pair(), multiple=True,
              help="Event attribute (key=value, multiple)")
@click.option('-m', '--metric', '--metric_f', type=click.FLOAT,
              help="Event metric (uses metric_f)")
@click.option('--echo/--no-echo', default=True,
              help="Echo event object after sending")
@click.pass_obj
def send(transport, time, state, host, description, service, tag, attribute,
         ttl, metric_f, echo):
    """Send a single event to Riemann"""
    client = CommandLineClient(transport)
    event = client.create_event({
        'time': time,
        'state': state,
        'host': host,
        'description': description,
        'service': service,
        'tags': tag,
        'attributes': dict(attribute),
        'ttl': ttl,
        'metric_f': metric_f
    })

    with client:
        client.send_event(event)
    if echo:
        echo_event(client.create_dict(event))


@main.command()
@click.argument('query', 'query')
@click.pass_obj
def query(transport, query):
    """Query the Riemann server"""
    with CommandLineClient(transport) as client:
        echo_event(client.query(query))



from __future__ import absolute_import

import py.test
import socket
import time

import riemann_client.client
import riemann_client.riemann_pb2
import riemann_client.transport


@py.test.fixture
def blank_transport():
    return riemann_client.transport.BlankTransport()


class BrokenTransport(riemann_client.transport.Transport):
    def connect(self):
        pass

    def disconnect(self):
        pass

    def send(self, *args, **kwargs):
        raise socket.error(32, '[Errno 32] Broken pipe')


@py.test.fixture
def broken_transport():
    return BrokenTransport()


@py.test.fixture
def auto_flushing_queued_client(request, blank_transport):
    """A Riemann client using the StringIO transport and
    AutoFlushingQueuedClient with max_delay=300 and
    max_batch_size=5000"""
    client = riemann_client.client.AutoFlushingQueuedClient(
        transport=blank_transport,
        max_delay=300,
        max_batch_size=5000,
        stay_connected=True)
    client.transport.connect()

    @request.addfinalizer
    def disconnect():
        client.transport.disconnect()

    return client


@py.test.fixture
def auto_flushing_queued_client_delay(request, blank_transport):
    """A Riemann client using the StringIO transport and
    AutoFlushingQueuedClient with max_delay=1 and
    max_batch_size=5000"""
    client = riemann_client.client.AutoFlushingQueuedClient(
        transport=blank_transport,
        max_delay=0.05,
        max_batch_size=5000,
        stay_connected=True)
    client.transport.connect()

    @request.addfinalizer
    def disconnect():
        client.transport.disconnect()

    return client


@py.test.fixture
def auto_flushing_queued_client_batch5(request, blank_transport):
    """A Riemann client using the StringIO transport and
    AutoFlushingQueuedClient with max_delay=300 and
    max_batch_size=5"""
    client = riemann_client.client.AutoFlushingQueuedClient(
        transport=blank_transport,
        max_delay=300,
        max_batch_size=5,
        stay_connected=True)
    client.transport.connect()

    @request.addfinalizer
    def disconnect():
        client.transport.disconnect()

    return client


@py.test.fixture
def auto_flushing_queued_client_batch5_broken_f(request, broken_transport):
    """A Riemann client using the StringIO transport and
    AutoFlushingQueuedClient with max_delay=300 and
    max_batch_size=5"""
    client = riemann_client.client.AutoFlushingQueuedClient(
        transport=broken_transport,
        max_delay=300,
        max_batch_size=5,
        stay_connected=True,
        clear_on_fail=False)
    client.transport.connect()

    @request.addfinalizer
    def disconnect():
        client.transport.disconnect()

    return client


@py.test.fixture
def auto_flushing_queued_client_batch5_broken_t(request, broken_transport):
    """A Riemann client using the StringIO transport and
    AutoFlushingQueuedClient with max_delay=300 and
    max_batch_size=5"""
    client = riemann_client.client.AutoFlushingQueuedClient(
        transport=broken_transport,
        max_delay=300,
        max_batch_size=5,
        stay_connected=True,
        clear_on_fail=True)
    client.transport.connect()

    @request.addfinalizer
    def disconnect():
        client.transport.disconnect()

    return client


@py.test.fixture
def using_simple_queue(auto_flushing_queued_client):
    """An event queue with a single event"""
    auto_flushing_queued_client.event(service='test')


@py.test.fixture
def large_queue(auto_flushing_queued_client):
    """An event queue with 100 events"""
    items = ['-->{0}<--'.format(i) for i in range(0, 1000)]
    for description in items:
        auto_flushing_queued_client.event(service='queue',
                                          description=description)
    return items


def test_simple_queue_event(auto_flushing_queued_client, using_simple_queue):
    assert auto_flushing_queued_client.queue.events[0].service == 'test'


def test_simple_queue_length(auto_flushing_queued_client, using_simple_queue):
    assert len(auto_flushing_queued_client.queue.events) == 1


def test_simple_queue_event_not_sent(auto_flushing_queued_client,
                                     using_simple_queue):
    assert len(auto_flushing_queued_client.transport.events) == 0


def test_simple_queue_event_sent(auto_flushing_queued_client,
                                 using_simple_queue):
    auto_flushing_queued_client.flush()
    assert len(auto_flushing_queued_client.transport.events) == 1


def test_deciqueue_length(auto_flushing_queued_client, large_queue):
    assert len(auto_flushing_queued_client.queue.events) == len(large_queue)


def test_deciqueue_output(auto_flushing_queued_client, large_queue):
    auto_flushing_queued_client.flush()
    # note that these will be ordered, and all events will be in a single
    # protobuf flush, so we can find them in messages[0]
    for idx, description in enumerate(large_queue):
        assert (description == auto_flushing_queued_client.
                transport.events[idx].description)


def test_deciqueue_flush(auto_flushing_queued_client, large_queue):
    auto_flushing_queued_client.flush()
    assert len(auto_flushing_queued_client.queue.events) == 0


def test_clear_queue(auto_flushing_queued_client, using_simple_queue):
    auto_flushing_queued_client.clear_queue()
    assert len(auto_flushing_queued_client.queue.events) == 0


def test_batchsize_autoflush(auto_flushing_queued_client_batch5):
    auto_flushing_queued_client_batch5.clear_queue()
    sent = 0
    to_send = 100
    for i in range(to_send):
        auto_flushing_queued_client_batch5.event(
            service='test', description='{0:03d}'.format(i))
        sent += 1
    assert len(auto_flushing_queued_client_batch5.queue.events) == 0
    assert len(auto_flushing_queued_client_batch5.transport) == sent
    assert ('000' == auto_flushing_queued_client_batch5.
            transport.events[0].description)
    assert ('{0:03d}'.format(to_send - 1) ==
            auto_flushing_queued_client_batch5.transport.
            events[-1].description)


def test_timer_autoflush(auto_flushing_queued_client_delay):
    auto_flushing_queued_client_delay.clear_queue()
    auto_flushing_queued_client_delay.event(service='test',
                                            description='timer_test')
    assert len(auto_flushing_queued_client_delay.transport) == 0
    assert len(auto_flushing_queued_client_delay.queue.events) == 1
    time.sleep(0.1)
    assert len(auto_flushing_queued_client_delay.transport) == 1
    assert len(auto_flushing_queued_client_delay.queue.events) == 0
    assert ('timer_test' == auto_flushing_queued_client_delay.
            transport.events[0].description)


def test_clear_on_fail_false(auto_flushing_queued_client_batch5_broken_f):
    auto_flushing_queued_client_batch5_broken_f.clear_queue()
    sent = 0
    to_send = 10
    for i in range(to_send):
        auto_flushing_queued_client_batch5_broken_f.event(
            service='test', description='{0:03d}'.format(i))
        sent += 1
    assert (len(auto_flushing_queued_client_batch5_broken_f.queue.events) ==
            sent)


def test_clear_on_fail_true(auto_flushing_queued_client_batch5_broken_t):
    auto_flushing_queued_client_batch5_broken_t.clear_queue()
    sent = 0
    to_send = 10
    for i in range(to_send):
        auto_flushing_queued_client_batch5_broken_t.event(
            service='test', description='{0:03d}'.format(i))
        sent += 1
    assert (len(auto_flushing_queued_client_batch5_broken_t.queue.events) ==
            0)



from __future__ import absolute_import

import re
import socket

import click.testing

import riemann_client.command


def run_cli(args):
    args = [u'-T', u'none'] + list(args)
    runner = click.testing.CliRunner()
    result = runner.invoke(riemann_client.command.main, args)
    assert result.exit_code == 0
    return result


def strip(string):
    return re.sub(u'\s+', u'', string)


def assert_output_eq(args, expected):
    assert strip(run_cli(args).output) == strip(expected)


def test_send_empty_message():
    assert_output_eq([u'send'], u'{"host": "%s"}' % socket.gethostname())


POPULATED_MESSAGE = u"""{
  "attributes": {
    "key": "value"
  },
  "description": "description",
  "host": "%s",
  "metric_f": 11.5,
  "service": "service",
  "state": "state",
  "tags": [
    "tag"
  ],
  "time": 1408030991,
  "ttl": 120.5
}
""" % socket.gethostname()


def test_send():
    assert_output_eq([
        u'send',
        u'--attribute', u'key=value',
        u'--description', u'description',
        u'--metric_f', u'11.5',
        u'--service', u'service',
        u'--state', u'state',
        u'--tag', u'tag',
        u'--time', u'1408030991',
        u'--ttl', u'120.5'
    ], POPULATED_MESSAGE)


def test_send_short():
    assert_output_eq([
        u'send',
        u'-a', u'key=value',
        u'-d', u'description',
        u'-m', u'11.5',
        u'-s', u'service',
        u'-S', u'state',
        u'-t', u'tag',
        u'-T', u'1408030991',
        u'-l', u'120.5'
    ], POPULATED_MESSAGE)


def test_send_noecho():
    assert_output_eq([
        'send',
        '--attribute', 'key=value',
        '--description', 'description',
        '--metric_f', '11.5',
        '--service', 'service',
        '--state', 'state',
        '--tag', 'tag',
        '--time', '1408030991',
        '--ttl', '120.5',
        '--no-echo'
    ], '')


def test_query():
    assert_output_eq([u'query', u'true'], u'[]')



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: riemann.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
import sys
if sys.version_info >= (3,):
  #some constants that are python2 only
  unicode = str
  long = int
  range = range
  unichr = chr
  def b(s):
    return s.encode("latin-1")
  def u(s):
    return s
else:
  #some constants that are python2 only
  range = xrange
  unicode = unicode
  long = long
  unichr = unichr
  def b(s):
    return s
  # Workaround for standalone backslash
  def u(s):
    return unicode(s.replace(r'\\', r'\\\\'), "unicode_escape")

from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)




DESCRIPTOR = _descriptor.FileDescriptor(
  name='riemann.proto',
  package='',
  serialized_pb=b('\n\rriemann.proto\"\x81\x01\n\x05State\x12\x0c\n\x04time\x18\x01 \x01(\x03\x12\r\n\x05state\x18\x02 \x01(\t\x12\x0f\n\x07service\x18\x03 \x01(\t\x12\x0c\n\x04host\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x0c\n\x04once\x18\x06 \x01(\x08\x12\x0c\n\x04tags\x18\x07 \x03(\t\x12\x0b\n\x03ttl\x18\x08 \x01(\x02\"\xce\x01\n\x05\x45vent\x12\x0c\n\x04time\x18\x01 \x01(\x03\x12\r\n\x05state\x18\x02 \x01(\t\x12\x0f\n\x07service\x18\x03 \x01(\t\x12\x0c\n\x04host\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x0c\n\x04tags\x18\x07 \x03(\t\x12\x0b\n\x03ttl\x18\x08 \x01(\x02\x12\x1e\n\nattributes\x18\t \x03(\x0b\x32\n.Attribute\x12\x15\n\rmetric_sint64\x18\r \x01(\x12\x12\x10\n\x08metric_d\x18\x0e \x01(\x01\x12\x10\n\x08metric_f\x18\x0f \x01(\x02\"\x17\n\x05Query\x12\x0e\n\x06string\x18\x01 \x01(\t\"g\n\x03Msg\x12\n\n\x02ok\x18\x02 \x01(\x08\x12\r\n\x05\x65rror\x18\x03 \x01(\t\x12\x16\n\x06states\x18\x04 \x03(\x0b\x32\x06.State\x12\x15\n\x05query\x18\x05 \x01(\x0b\x32\x06.Query\x12\x16\n\x06\x65vents\x18\x06 \x03(\x0b\x32\x06.Event\"\'\n\tAttribute\x12\x0b\n\x03key\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x01(\tB\x1a\n\x11\x63om.aphyr.riemannB\x05Proto'))




_STATE = _descriptor.Descriptor(
  name='State',
  full_name='State',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='State.time', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state', full_name='State.state', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode(b(""), "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='service', full_name='State.service', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode(b(""), "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='host', full_name='State.host', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode(b(""), "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='State.description', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode(b(""), "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='once', full_name='State.once', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tags', full_name='State.tags', index=6,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ttl', full_name='State.ttl', index=7,
      number=8, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=18,
  serialized_end=147,
)


_EVENT = _descriptor.Descriptor(
  name='Event',
  full_name='Event',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='Event.time', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state', full_name='Event.state', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode(b(""), "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='service', full_name='Event.service', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode(b(""), "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='host', full_name='Event.host', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode(b(""), "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='Event.description', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode(b(""), "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tags', full_name='Event.tags', index=5,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ttl', full_name='Event.ttl', index=6,
      number=8, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='attributes', full_name='Event.attributes', index=7,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metric_sint64', full_name='Event.metric_sint64', index=8,
      number=13, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metric_d', full_name='Event.metric_d', index=9,
      number=14, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metric_f', full_name='Event.metric_f', index=10,
      number=15, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=150,
  serialized_end=356,
)


_QUERY = _descriptor.Descriptor(
  name='Query',
  full_name='Query',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='string', full_name='Query.string', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode(b(""), "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=358,
  serialized_end=381,
)


_MSG = _descriptor.Descriptor(
  name='Msg',
  full_name='Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ok', full_name='Msg.ok', index=0,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='error', full_name='Msg.error', index=1,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode(b(""), "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='states', full_name='Msg.states', index=2,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='query', full_name='Msg.query', index=3,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='events', full_name='Msg.events', index=4,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=383,
  serialized_end=486,
)


_ATTRIBUTE = _descriptor.Descriptor(
  name='Attribute',
  full_name='Attribute',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='Attribute.key', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode(b(""), "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='Attribute.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode(b(""), "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=488,
  serialized_end=527,
)

_EVENT.fields_by_name['attributes'].message_type = _ATTRIBUTE
_MSG.fields_by_name['states'].message_type = _STATE
_MSG.fields_by_name['query'].message_type = _QUERY
_MSG.fields_by_name['events'].message_type = _EVENT
DESCRIPTOR.message_types_by_name['State'] = _STATE
DESCRIPTOR.message_types_by_name['Event'] = _EVENT
DESCRIPTOR.message_types_by_name['Query'] = _QUERY
DESCRIPTOR.message_types_by_name['Msg'] = _MSG
DESCRIPTOR.message_types_by_name['Attribute'] = _ATTRIBUTE

State = _reflection.GeneratedProtocolMessageType('State', (_message.Message,),
    {
      'DESCRIPTOR': _STATE,
      # @@protoc_insertion_point(class_scope:State)
    })

Event = _reflection.GeneratedProtocolMessageType('Event', (_message.Message,),
    {
      'DESCRIPTOR': _EVENT,
      # @@protoc_insertion_point(class_scope:Event)
    })

Query = _reflection.GeneratedProtocolMessageType('Query', (_message.Message,),
    {
      'DESCRIPTOR': _QUERY,
      # @@protoc_insertion_point(class_scope:Query)
    })

Msg = _reflection.GeneratedProtocolMessageType('Msg', (_message.Message,),
    {
      'DESCRIPTOR': _MSG,
      # @@protoc_insertion_point(class_scope:Msg)
    })

Attribute = _reflection.GeneratedProtocolMessageType('Attribute', (_message.Message,),
    {
      'DESCRIPTOR': _ATTRIBUTE,
      # @@protoc_insertion_point(class_scope:Attribute)
    })


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), b('\n\021com.aphyr.riemannB\005Proto'))
# @@protoc_insertion_point(module_scope)



from __future__ import absolute_import

import py.test

import riemann_client.riemann_pb2
import riemann_client.transport

from riemann_client.transport import socket_recvall


class FakeSocket(object):
        def __init__(self):
            self.data = [b'hello', b'world', b'']

        def recv(self, bufsize):
            return self.data.pop(0)


def test_socket_recvall():
    assert socket_recvall(FakeSocket(), 10) == b'helloworld'


def test_socket_recvall_short():
    assert socket_recvall(FakeSocket(), 5) == b'hello'


@py.test.fixture
def tcp_transport():
    return riemann_client.transport.TCPTransport()


def test_not_yet_connected(tcp_transport):
    with py.test.raises(RuntimeError):
        tcp_transport.send(riemann_client.riemann_pb2.Msg())


def test_address_property(tcp_transport):
    assert tcp_transport.address == (tcp_transport.host, tcp_transport.port)


def test_transport_enter(string_transport):
    assert not hasattr(string_transport, 'string')
    with string_transport:
        assert hasattr(string_transport, 'string')
